/**
 * \mainpage 10 komplex szám abszolútértéke
 *
 * \section feladat Feladat
 * Olvassunk be max 10 komplex számot, majd
 * írjuk ki a számokat és abszolútértéküket fordított sorrendben!
 *
 * \section megv Megvalósítás
 * Megvalósítás objektum-orientált dekompozícióval és
 * operátor overload használatával.
 *
 * \section korl Korlátozás, hiányosságok, kiegészítés
 * Nincs indexhatárellenőrzés.
 * Összedás művelet tesztjei és a cast hatásának demonstrálása
 *
 * \section fordítási opciók
 * Ha az OP_K_PLUS_D azonosító nincs definiálva, akkor
 *   nem keletkezik double + komplex fv.
 *   Ekkor az automatikus típuskonverziók miatt illeszkedni
 *   fog a (double) cast operator az int + Komplex kifejezésre.
 *
 * Ha a TEST_K_PLUS_I azonosító definiálva van, akkor
 *   Komplex + int művelet is lesz, ami viszont az automatikus
 *   típuskonverziók miatt beindítja a (double) + int beépített
 *   műveletet. Ez viszont a (double) cast operator miatt fog illeszkedni.
 *
 */

#define OP_K_PLUS_D
//#define TEST_K_PLUS_I

#include <iostream>
#include <cmath>

using std::cout;
using std::endl;
using std::ostream;
using std::cin;
using std::istream;

/**
 * Komplex osztály.
 * Komplex viselkedést megvalósító osztály.
 * Csak a feladat megoldásához szükséges műveleteket definiáltuk.
 */
class Komplex {
	double re;		// valós rész
	double im;		// képzetes rész
public:
    Komplex() { re = 0; im = 0; }  // paraméter nélküli konstruktor
    Komplex(double r, double i = 0) { re = r; im = i; } // 1 és 2 paraméteres konstruktor
	// setter függvények
	void setRe(double d)    { re = d; }
    void setIm(double i)    { im = i; }
	// getter függvények
	double getRe() const    { return re; }
	double getIm() const    { return im; }
	// cast operator
    operator double() const { return sqrt(re*re + im*im); }   ///< abszolútérték
    // összeadások
    Komplex operator+(const Komplex& k) const { 	// komplex + komplex
        Komplex sum(k.re + re, k.im + im);
        return sum;
    }
#ifdef OP_K_PLUS_D
    Komplex operator+(const double r) const { 	// komplex + double
        return *this + Komplex(r); 		        // visszavezetjük komplex + komplexre
    }
#endif
};

/**
 * Globális operátor fv. a double + komplex összeadásához
 */
Komplex operator+(double d, const Komplex& k) {	// double + komplex
	return k + d;
}

/**
 * Globális operátor fv. a beolvasáshoz
 */
istream& operator>>(istream& s, Komplex& k) {
		double re, im;
        s >> re >> im;
		k.setRe(re);
		k.setIm(im);
		return(s);
}

/**
 * Globális operátor fv. a kiíráshoz
 * friend, hogy közvetlenül elérhesse a privát adattagokat
 */
ostream& operator<<(ostream& s, const Komplex k) {
        s << k.getRe() << ' ' << k.getIm(); return(s);
}

/**
 * KomplexTar osztály.
 * Tároló ami max 10 komplex számot tud tárolni
 * Az egyszerűség kedvéért nincs indexhatár ellenűrzés!
 */
class KomplexTar {
    Komplex t[10];      // belső objektumtömböt tartalmaz a tároláshoz
public:
    Komplex& operator[](const int i)  { return t[i]; }  ///< indexelés, ami referenciát ad, hogy balérték is lehessen.
};


/**
 * Főprogram.
 */
int main()
{
    KomplexTar t;       // tároló létrehozása
    int i;              // ciklusváltozó és darab

    for (i = 0; i < 10 && cin >> t[i]; i++); // beolvas és tárol

    for (i = i-1; i >= 0; i--) {
        Komplex k = t[i]; // definició és inic.
        cout << k << ' ' << (double)k << endl; // kiír a megfelelő formátumban
    }

// Komplex összeadás tesztek
    Komplex k1, k2(1, 0);	// konstruktorok próbája
    cout << k1 << endl;		// kiírás teszt
    cout << k2 << endl;
    cout << k2+k2 << endl;	// komplex + komplex
    cout << k2+1.0 << endl;	// komplex + double
    cout << 1.0+k2 << endl;	// double + komplex
#ifdef TEST_K_PLUS_I
    cout << k2+1;	        // a cast miatt nem egyértelmű
#endif
    return 0;
}
